# -*- coding: utf-8 -*-

# © 2020 Le Filament (<http://www.le-filament.com>)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html).

from datetime import datetime

from odoo import fields, models, api
from odoo.exceptions import UserError,Warning


class AccountInvoicePcaFae(models.Model):
    _inherit = 'account.invoice'

    invoice_pcafae_line_ids = fields.One2many(
        comodel_name='account.invoice.pcafae.line',
        inverse_name='invoice_id',
        string='Pondération annuelle'
    )
    fiscal_year = fields.Char(
        string='Année Fiscale de la facture',
        compute='_get_fiscal_year',
        store=True
    )
    is_weighted = fields.Boolean(
        string='Ventilé',
        compute='_compute_is_weighted',
        store=True,
        default=False
    )
    pca_is_exported = fields.Boolean("PCA exporté ?", default=False)
    pca_export_date = fields.Datetime("Date export PCA")

    # ------------------------------------------------------
    # Constrains function
    # ------------------------------------------------------
    @api.constrains('invoice_pcafae_line_ids')
    def _check_total_percent(self):
        total = sum(self.invoice_pcafae_line_ids.mapped('year_percent'))
        if total != 100.0 and self.invoice_pcafae_line_ids:
            raise UserError("Le total des pourcentages doit être égal à 100 !")
        if self.invoice_pcafae_line_ids and not str(self.fiscal_year) in self.invoice_pcafae_line_ids.mapped('year'):
            raise UserError("L'année fiscale de la facture doit être présente dans la ventilation (à 0% si nécessaire).")
        return True

    # ------------------------------------------------------
    # Compute function
    # ------------------------------------------------------
    @api.depends('date_invoice')
    @api.multi
    def _get_fiscal_year(self):
        """
        Calcule l'année fiscale de la facture en fonction de la date de cloture
        :return: fiscal_year (char)
        """
        for invoice in self:
            if invoice.date_invoice:
                date_real = datetime.strptime(invoice.date_invoice, '%Y-%m-%d')
                end_fiscal_year = datetime(
                    date_real.year,
                    invoice.company_id.fiscalyear_last_month,
                    invoice.company_id.fiscalyear_last_day
                )
                if date_real > end_fiscal_year:
                    invoice.fiscal_year = str(date_real.year + 1)
                else:
                    invoice.fiscal_year = str(date_real.year)

    @api.multi
    @api.depends('invoice_pcafae_line_ids')
    def _compute_is_weighted(self):
        for inv in self:
            if inv.invoice_pcafae_line_ids:
                inv.is_weighted = True
            else:
                inv.is_weighted = False

    # ------------------------------------------------------
    # Override ORM
    # ------------------------------------------------------
    @api.multi
    def write(self, values):
        """
        Calcule le PCA et FAE N-1 pour l'année en cours par rapport à
        l'ensemble des lignes pondérées
        :param values:
        :return: self
        """
        res = super(AccountInvoicePcaFae, self).write(values)
        if values.get('invoice_pcafae_line_ids', False):
            if len(self.invoice_pcafae_line_ids) > 1:
                line_fy_id = self.invoice_pcafae_line_ids.filtered(
                    lambda y: y.year == str(self.fiscal_year))
                line_ids = self.invoice_pcafae_line_ids - line_fy_id
                for line in line_ids:
                    # si FAE, on affecte le FAE négatif sur l'année en cours
                    if line.fae:
                        line_fy_id.fae_last_year = -line.fae
                    # si PCA, on affecte le PCA négatif sur l'année en cours
                    elif line.pca_last_year:
                        line_fy_id.pca = -line.pca_last_year
        return res
