# -*- coding: utf-8 -*-

# © 2020 Le Filament (<http://www.le-filament.com>)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html).

import re
from datetime import datetime

from odoo import fields, models, api


class AccountInvoicePcaFaeLines(models.Model):
    _name = 'account.invoice.pcafae.line'
    _description = 'Lignes de PCA'
    _order = 'year, partner_id'

    year = fields.Char(
        string='Année Fiscale',
        default=str(datetime.now().year),
        required=True
    )
    invoice_id = fields.Many2one(
        comodel_name='account.invoice',
        string='Facture',
    )
    partner_id = fields.Many2one(
        comodel_name='res.partner',
        related='invoice_id.partner_id',
        string='Client',
        store=True
    )
    date = fields.Date(
        string='Date Facture',
        related='invoice_id.date_invoice',
        store=True
    )
    invoice_state = fields.Selection(
        string='Status',
        related='invoice_id.state',
        store=True
    )
    company_id = fields.Many2one(
        comodel_name='res.company',
        string='Company',
        change_default=True, required=True, readonly=True,
        default=lambda self: self.env['res.company']._company_default_get(
            'account.invoice.pcafae.line'))
    currency_id = fields.Many2one(
        comodel_name='res.currency',
        related='company_id.currency_id',
        string="Company Currency",
        readonly=True
    )
    year_percent = fields.Float('Pourcentage', default=0.0)
    # Invoice computed fields
    fae = fields.Float(
        'FAE',
        currency_field='company_currency_id',
        compute='_compute_invoice',
        store=True,
        default=0.0)
    fae_last_year = fields.Float(
        'FAE N-1',
        currency_field='company_currency_id',
        compute='_compute_invoice',
        store=True,
        default=0.0)
    pca = fields.Float(
        'PCA',
        currency_field='company_currency_id',
        compute='_compute_invoice',
        store=True,
        default=0.0)
    pca_last_year = fields.Float(
        'PCA N-1',
        currency_field='company_currency_id',
        compute='_compute_invoice',
        store=True,
        default=0.0)
    ca = fields.Float(
        'CA',
        currency_field='company_currency_id',
        compute='_compute_invoice',
        store=True,
        default=0.0)
    real = fields.Float(
        'Facturé réel',
        currency_field='company_currency_id',
        compute='_compute_invoice',
        store=True,
        default=0.0)

    # ------------------------------------------------------
    # Contraintes SQL
    # ------------------------------------------------------
    # Unicité Année - Facture
    _sql_constraints = [
        ('year_unique',
         'unique(invoice_id, year)',
         "Il ne peut y avoir qu'une seule ligne de ventilation par année sur la facture"),
    ]

    # ------------------------------------------------------
    # Compute function
    # ------------------------------------------------------
    @api.depends(
        'year_percent',
        'invoice_id.amount_untaxed_signed',
        'invoice_id.fiscal_year',)
    @api.multi
    def _compute_invoice(self):
        for line in self:
            line.ca = (0.01 * line.year_percent *
                       line.invoice_id.amount_untaxed_signed)
            if line.year < line.invoice_id.fiscal_year:
                line.fae = line.ca
            elif line.year == line.invoice_id.fiscal_year:
                line.real = line.invoice_id.amount_untaxed_signed
            elif line.year > line.invoice_id.fiscal_year:
                line.pca_last_year = line.ca

    # ------------------------------------------------------
    # Onchange function
    # ------------------------------------------------------
    @api.onchange('year')
    def _onchange_year(self):
        valid_year = re.match('[2][0][1-4][0-9]', self.year)
        if not valid_year:
            return {
                'warning': {
                    'title': 'Erreur Valeur',
                    'message': "L'année n'a pas une valeur valide"
                }
            }
